package cn.iocoder.yudao.module.member.mq.consumer.score.core;

import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQuery;
import cn.iocoder.yudao.module.member.dal.dataobject.scoreRule.ScoreRuleDO;
import cn.iocoder.yudao.module.member.dal.redis.scoreRule.ScoreRuleRedisDao;
import cn.iocoder.yudao.module.member.enums.ScoreRuleStatusEnum;
import cn.iocoder.yudao.module.member.enums.ScoreRuleTypeEnum;
import cn.iocoder.yudao.module.member.enums.TransportTypeEnum;
import cn.iocoder.yudao.module.member.service.scoreRule.ScoreRuleService;
import org.springframework.beans.factory.annotation.Autowired;

import javax.annotation.Resource;
import java.util.Date;

/**
 * @author zhangfeng
 */
public abstract class AbstractScoreRuleStrategy implements ScoreRuleStrategy {

    protected ScoreRuleService scoreRuleService;

    @Autowired
    public AbstractScoreRuleStrategy(ScoreRuleService scoreRuleService) {
        this.scoreRuleService = scoreRuleService;
    }

    @Resource
    ScoreRuleRedisDao scoreRuleRedisDao;

    public ScoreRuleDO getEnableScoreRuleByType(ScoreRuleTypeEnum scoreRuleType) {
        ScoreRuleDO enableScoreRule;
        enableScoreRule = scoreRuleRedisDao.getEnableScoreRule(scoreRuleType);
        if (enableScoreRule == null) {
            enableScoreRule = scoreRuleService.selectOne(new LambdaQuery<ScoreRuleDO>()
                    .eq(ScoreRuleDO::getType, scoreRuleType.getValue())
                    .eq(ScoreRuleDO::getStatus, ScoreRuleStatusEnum.ENABLED.getValue()));
            if (enableScoreRule == null) {
                return null;
            }
            // 校验一下活动开始结束时间
            if (!enableScoreRule.getEndTime().after(new Date()) || !enableScoreRule.getStartTime().before(new Date())) {
                return null;
            }
            scoreRuleRedisDao.setEnableScoreRule(enableScoreRule);
        } else {
            if (!enableScoreRule.getEndTime().after(new Date()) || !enableScoreRule.getStartTime().before(new Date())) {
                return null;
            }
        }
        return enableScoreRule;
    }

    public ScoreRuleDO getEnableScoreRuleByType(ScoreRuleTypeEnum scoreRuleType, TransportTypeEnum transportType) {
        ScoreRuleDO enableScoreRule;
        switch (scoreRuleType) {
            case ORDER_V:
                enableScoreRule = scoreRuleRedisDao.getEnableScoreRule(scoreRuleType, transportType);
                if (enableScoreRule == null) {
                    enableScoreRule = scoreRuleService.getEnabledOrderVScoreRuleByTransportType(transportType.getValue());
                    if (enableScoreRule == null) {
                        return null;
                    }
                    if (!enableScoreRule.getEndTime().after(new Date()) || !enableScoreRule.getStartTime().before(new Date())) {
                        return null;
                    }
                    scoreRuleRedisDao.setEnableScoreRule(enableScoreRule);
                } else {
                    if (!enableScoreRule.getEndTime().after(new Date()) || !enableScoreRule.getStartTime().before(new Date())) {
                        return null;
                    }
                }
                return enableScoreRule;
            case RECOMMEND:
            case REGISTER:
                return getEnableScoreRuleByType(scoreRuleType);
        }
        return null;
    }
}

