package cn.iocoder.yudao.module.member.mq.consumer.score.core;

import cn.hutool.core.collection.CollUtil;
import cn.iocoder.yudao.module.member.api.score.MemberUserScoreApi;
import cn.iocoder.yudao.module.member.api.score.dto.MemberUserScoreOperateReqDTO;
import cn.iocoder.yudao.module.member.dal.dataobject.scoreLog.MemberUserScoreLogDO;
import cn.iocoder.yudao.module.member.dal.dataobject.scoreRule.ScoreRuleDO;
import cn.iocoder.yudao.module.member.dal.dataobject.user.MemberUserDO;
import cn.iocoder.yudao.module.member.enums.ScoreRuleTypeEnum;
import cn.iocoder.yudao.module.member.enums.ScoreSourceTypeEnum;
import cn.iocoder.yudao.module.member.mq.message.ScoreMessage;
import cn.iocoder.yudao.module.member.service.scoreLog.MemberUserScoreLogService;
import cn.iocoder.yudao.module.member.service.scoreRule.ScoreRuleService;
import cn.iocoder.yudao.module.member.service.user.MemberUserService;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;

/**
 * @author zhangfeng
 */
@Service
@Slf4j
public class RecommendStrategy extends AbstractScoreRuleStrategy {
    @Resource
    private MemberUserScoreApi memberUserScoreApi;
    @Resource
    private MemberUserService memberUserService;
    @Resource
    private MemberUserScoreLogService memberUserScoreLogService;

    public RecommendStrategy(ScoreRuleService scoreRuleService) {
        super(scoreRuleService);
    }

    @Override
    public void addScore(ScoreMessage message) {
        Long userId = message.getUserId();
        log.info("Received Recommend message,userID:{}", userId);
        ScoreRuleDO scoreRuleDO = getEnableScoreRuleByType(ScoreRuleTypeEnum.RECOMMEND);
        if (scoreRuleDO == null) {
            log.info("Recommend rule is not exist,userID:{}", userId);
            return;
        }
        MemberUserDO memberUserDO = memberUserService.getUser(userId);
        if (memberUserDO == null) {
            log.info("Recommend listener: The user does not exist,userID:{}", userId);
            return;
        }
        // 校验累计最高分
        Integer userScoreTotalCount = 0;
        LambdaQueryWrapper<MemberUserScoreLogDO> scoreLogQueryWrapper = new LambdaQueryWrapper<>();
        scoreLogQueryWrapper.eq(MemberUserScoreLogDO::getMemberId, userId)
                .eq(MemberUserScoreLogDO::getRuleId, scoreRuleDO.getId());
        List<MemberUserScoreLogDO> memberUserScoreLogs = memberUserScoreLogService.selectList(scoreLogQueryWrapper);
        if (CollUtil.isNotEmpty(memberUserScoreLogs)) {
            for (MemberUserScoreLogDO memberUserScoreLog : memberUserScoreLogs) {
                userScoreTotalCount += memberUserScoreLog.getScoreCount();
            }
        }
        if (userScoreTotalCount >= scoreRuleDO.getMaxScoreTotal()) {
            log.info("Recommend listener: The user has reached the maximum score,userId:{},scoreRuleId:{}", userId, scoreRuleDO.getId());
            return;
        }
        try {
            HashMap<String, Object> map = new HashMap<>();
            map.put("userId", userId);
            map.put("userNameZh", message.getUserNameZh());
            map.put("userNameEn", message.getUserNameEn());
            String desensitizedPhoneNumber = message.getPhone().replaceAll("(\\d{3})\\d{4}(\\d{4})", "$1****$2");
            map.put("phone", desensitizedPhoneNumber);

            memberUserScoreApi.operateScore(MemberUserScoreOperateReqDTO.builder()
                    .memberId(userId)
                    .scoreCount(scoreRuleDO.getGetScoreOnce())
                    .sourceType(ScoreSourceTypeEnum.RECOMMEND)
                    .ruleId(scoreRuleDO.getId())
                    .uniqueId(userId + "_" + message.getRecommendUserId())
                    .expireDays(scoreRuleDO.getScorePeriod())
                    .build());
        } catch (Exception e) {
            log.error("Recommend listener: operateScore error", e);
        }
        log.info("Recommend rule add score success,ruleID:{},userID:{}", scoreRuleDO.getId(), userId);
    }

    @Override
    public ScoreRuleTypeEnum getStrategyScoreRuleType() {
        return ScoreRuleTypeEnum.RECOMMEND;
    }
}
